@@ -206,6 +206,14 @@ class AzureDatalakeFileSystem(AbstractFileSystem):
     def size(self, path):
         return self.info(path)["length"]
 
+    def rmdir(self, path):
+        """Remove a directory, if empty"""
+        self.azure_fs.rmdir(path)
+
+    def rm_file(self, path):
+        """Delete a file"""
+        self.azure_fs.rm(path)
+
     def __getstate__(self):
         dic = self.__dict__.copy()
         logger.debug("Serialize with state: %s", dic)
@@ -729,14 +737,18 @@ class AzureBlobFileSystem(AsyncFileSystem):
         return_glob: bool
 
         """
-        logger.debug(f"abfs.ls() is searching for {path}")
+        logger.debug("abfs.ls() is searching for %s", path)
         target_path = path.strip("/")
         container, path = self.split_path(path)
 
         if invalidate_cache:
             self.dircache.clear()
+
+        cache = {}
+        cache.update(self.dircache)
+
         if (container in ["", ".", "*", delimiter]) and (path in ["", delimiter]):
-            if _ROOT_PATH not in self.dircache or invalidate_cache or return_glob:
+            if _ROOT_PATH not in cache or invalidate_cache or return_glob:
                 # This is the case where only the containers are being returned
                 logger.info(
                     "Returning a list of containers in the azure blob storage account"
@@ -744,11 +756,12 @@ class AzureBlobFileSystem(AsyncFileSystem):
                 contents = self.service_client.list_containers(include_metadata=True)
                 containers = [c async for c in contents]
                 files = await self._details(containers)
-                self.dircache[_ROOT_PATH] = files
+                cache[_ROOT_PATH] = files
 
-            return self.dircache[_ROOT_PATH]
+            self.dircache.update(cache)
+            return cache[_ROOT_PATH]
         else:
-            if target_path not in self.dircache or invalidate_cache or return_glob:
+            if target_path not in cache or invalidate_cache or return_glob:
                 if container not in ["", delimiter]:
                     # This is the case where the container name is passed
                     async with self.service_client.get_container_client(
@@ -807,9 +820,10 @@ class AzureBlobFileSystem(AsyncFileSystem):
                         if not await self._exists(target_path):
                             raise FileNotFoundError
                         return []
+                    cache[target_path] = finalblobs
                     self.dircache[target_path] = finalblobs
 
-            return self.dircache[target_path]
+            return cache[target_path]
 
     async def _details(
         self,
@@ -881,7 +895,7 @@ class AzureBlobFileSystem(AsyncFileSystem):
             if len(output) == 1 and output[0]["type"] == "file":
                 # This handles the case where path is a file passed to ls
                 return output
-            output = await filter_blobs(output, target_path)
+            output = await filter_blobs(output, target_path, delimiter)
 
         return output
 
@@ -904,8 +918,8 @@ class AzureBlobFileSystem(AsyncFileSystem):
             Only return files that match `^{path}/{prefix}`
         kwargs are passed to ``ls``.
         """
-        path = self._strip_protocol(path)
-        parent_path = path.strip("/") + "/"
+        full_path = self._strip_protocol(path)
+        parent_path = full_path.strip("/") + "/"
         target_path = f"{parent_path}{(prefix or '').lstrip('/')}"
         container, path = self.split_path(target_path)
 
@@ -921,26 +935,33 @@ class AzureBlobFileSystem(AsyncFileSystem):
         detail = kwargs.pop("detail", False)
         try:
             infos = await self._details([b async for b in blobs])
-            for info in infos:
-                name = info["name"]
-                if name == target_path:
-                    continue
-                parent_dir = self._parent(name).rstrip("/") + "/"
-                if parent_dir not in dir_set and parent_dir != parent_path.strip("/"):
-                    dir_set.add(parent_dir)
-                    dirs[parent_dir] = {
-                        "name": parent_dir,
-                        "type": "directory",
-                        "size": 0,
-                    }
-                if info["type"] == "directory":
-                    dirs[name] = info
-                if info["type"] == "file":
-                    files[name] = info
-
         except ResourceNotFoundError:
             # find doesn't raise but returns [] or {} instead
-            pass
+            infos = []
+
+        for info in infos:
+            name = info["name"]
+            parent_dir = self._parent(name).rstrip("/") + "/"
+            if parent_dir not in dir_set and parent_dir != parent_path.strip("/"):
+                dir_set.add(parent_dir)
+                dirs[parent_dir] = {
+                    "name": parent_dir,
+                    "type": "directory",
+                    "size": 0,
+                }
+            if info["type"] == "directory":
+                dirs[name] = info
+            if info["type"] == "file":
+                files[name] = info
+
+        if not infos:
+            try:
+                file = await self._info(full_path)
+            except FileNotFoundError:
+                pass
+            else:
+                files[file["name"]] = file
+
         if withdirs:
             if not with_parent:
                 dirs.pop(target_path, None)
@@ -1277,8 +1298,19 @@ class AzureBlobFileSystem(AsyncFileSystem):
                 return True
 
         async with self.service_client.get_blob_client(container_name, path) as bc:
-            exists = await bc.exists()
-        return exists
+            if await bc.exists():
+                return True
+
+        dir_path = path.rstrip("/") + "/"
+        async with self.service_client.get_container_client(
+            container=container_name
+        ) as container_client:
+            async for blob in container_client.list_blobs(
+                results_per_page=1, name_starts_with=dir_path
+            ):
+                return True
+            else:
+                return False
 
     async def _pipe_file(self, path, value, overwrite=True, **kwargs):
         """Set the bytes of given file"""
@@ -1294,6 +1326,24 @@ class AzureBlobFileSystem(AsyncFileSystem):
 
     pipe_file = sync_wrapper(_pipe_file)
 
+    async def _cat_file(self, path, start=None, end=None, **kwargs):
+        path = self._strip_protocol(path)
+        if end is not None:
+            start = start or 0  # download_blob requires start if length is provided.
+            length = end - start
+        else:
+            length = None
+        container_name, path = self.split_path(path)
+        async with self.service_client.get_blob_client(
+            container=container_name, blob=path
+        ) as bc:
+            try:
+                stream = await bc.download_blob(offset=start, length=length)
+            except ResourceNotFoundError as e:
+                raise FileNotFoundError from e
+            result = await stream.readall()
+            return result
+
     def cat(self, path, recursive=False, on_error="raise", **kwargs):
         """Fetch (potentially multiple) paths' contents
         Returns a dict of {path: contents} if there are multiple paths
